local addon, ns = ...
local Cursor = DongleStub("Dongle-1.2"):New("Cursor")
_G.Cursor = Cursor

local tinsert = table.insert
local tremove = table.remove
local GetTime = Inspect.Time.Frame
local Mouse = Inspect.Mouse
local ctx = UI.CreateContext("Cursor")
local pos = {}
local moving = false
local fading = false
local lastUpdate = 0
local lastFadeUpdate = 0
local changeTexture -- up value function
local update
local enabled = true
local fileMap = {}

local frames = setmetatable({}, {__index=function(this, k)
    local v = rawget(this, k)
    if not v then 
        v = UI.CreateFrame("Texture", "Cursor."..k, ctx)
        v:SetLayer(k)
        rawset(this, k, v)
    end
    return v
end})

if FooBar then
    local mod = FooBarModule:new(FooBar.getFoobar(), "CursorFooBar")
    mod:setText("Cursor")
    mod:registerEvent("LeftClick", function() Cursor:OpenConfig() end)
    Cursor.foobar = mod
end

local defaults = {
    profile = {
        file = "light_green.png",
        points = 10,
        size = 1,
        r = 1,
        g = 1,
        b = 1,
        a = .3,
        applyBackground = false
    }
}
local function stopMoving()
    for i = 1, #pos do
        tremove(pos, 1)
    end
    for i, frame in ipairs(frames) do
        frame:SetVisible(false)
    end 
    moving = false
end

local lastFadeUpdate = 0
tinsert(Event.System.Update.Begin, {function()
    if #pos == 0 then fading = false end
    if not fading or moving then return end
    local now = GetTime()
    local delta = now - lastFadeUpdate
    if delta < .01 then return end
    lastFadeUpdate = now
    
    for i = 1, #pos do
        if #pos > Cursor.db.profile.points then
            tremove(pos, 1)
        end
    end
    
    tremove(pos, 1)

    for i, frame in ipairs(frames) do
        frame:SetVisible(false)
    end
    
    moving = true
    lastUpdate = now
    update[1](true)
    moving = false
    
end, "Cursor", "Fade out"})


update = {function(pass)
        if not Cursor.db or not moving then return end
        local points = math.floor(Cursor.db.profile.points)
        local size = Cursor.db.profile.size
        local now = GetTime()
        local delta = now - lastUpdate
        if delta > .1 and moving and not pass then
            fading = true
            moving = false
            lastUpdate = now
            return
        elseif not moving and delta < .1 and not pass then
            fading = true
            return
        end

        for i = 1, points do
                local p = pos[#pos-points+i]
                if p then
                    
                    local size = i * (size + 2)
                    if not moving and lastCount < i then
                        size = 0
                    end
                    frames[i]:SetPoint("TOPLEFT", UIParent, "TOPLEFT", p.x - size/2, p.y - size/2)
                    frames[i]:SetWidth(size)
                    frames[i]:SetHeight(size)
                    frames[i]:SetVisible(true)
                    if Cursor.db.profile.applyBackground then
                        local r = Cursor.db.profile.r
                        local g = Cursor.db.profile.g
                        local b = Cursor.db.profile.b
                        local a = Cursor.db.profile.a
                        frames[i]:SetBackgroundColor(r, g, b, a)
                    else
                        frames[i]:SetBackgroundColor(1, 1, 1, 0)
                    end
                end
        end
end, "Cursor", "Update"}
tinsert(Event.System.Update.Begin, update)

tinsert(Event.Mouse.Move, {function()
    lastUpdate = GetTime()
    local mouse = Mouse()
    tinsert(pos, {x=mouse.x, y=mouse.y})
    if not moving then
        stopMoving()
        moving = true
    end
end, "Cursor", "Event.Mouse.Move"})

--[[
do
    local lastX, lastY = 0, 0
    tinsert(Event.System.Update.Begin, {function()
        if not Cursor.db or not enabled then return end
        local mouse = Mouse()
        if mouse.x ~= lastX or mouse.y ~= lastY then
            tinsert(pos, {x=mouse.x, y = mouse.y})
            lastUpdate = GetTime()
            lastX, lastY = mouse.x, mouse.y
            if not moving then
                stopMoving()
                moving = true
            end
        end
    end, "Cursor", "Update.Mouse.Position"})
end
]]

local window = UI.CreateFrame("RiftWindow", "Config", ctx)
window:SetPoint("CENTER", UIParent, "CENTER")
window:SetHeight(300)
window:SetWidth(400)
window:SetVisible(false)

window.close = UI.CreateFrame("RiftButton", "Close", window)
window.close:SetText("Close Dialog")
window.close:SetPoint("BOTTOMCENTER", window, "BOTTOMCENTER", 0, -20)
window.close.Event.LeftClick = function()
    window:SetVisible(false)
end

function Cursor:OpenConfig()
    window:SetVisible(true)
end

local toggleChain = {}
local currentCheckbox
local firstToggle = true
local function createToggle(x, y, file)
    local tg = UI.CreateFrame("Frame", "Toggle", window)
    tg:SetHeight(12)
    tg:SetWidth(24)
    tg:SetPoint("TOPLEFT", window, "TOPLEFT", x + 20, y + 40)
    tg:SetBackgroundColor(.5, .8, .5, .1)
    
    tg.label = UI.CreateFrame("Texture", "Label", tg)
    tg.label:SetHeight(12)
    tg.label:SetWidth(12)
    tg.label:SetTexture("Cursor", file)
    tg.label:SetPoint("TOPLEFT", tg, "TOPLEFT", 0, 0)
    
    tg.checkbox = UI.CreateFrame("RiftCheckbox", "Checkbox", window)
    tg.checkbox:SetHeight(12)
    tg.checkbox:SetWidth(12)
    tg.checkbox:SetPoint("TOPLEFT", tg.label, "TOPRIGHT")
    tg.checkbox.Event.CheckboxChange = function()
        if tg.checkbox:GetChecked() then
            if currentCheckbox then currentCheckbox.checkbox:SetChecked(false) end
            currentCheckbox = tg
            changeTexture(tg)
        end
    end
    tg.file = file
    tinsert(toggleChain, tg)
    fileMap[file] = tg
    return tg
end

function changeTexture(tg)
    for i = 1, Cursor.db.profile.points do
        frames[i]:SetTexture("Cursor", tg.file)
    end
    if Cursor.db then
        Cursor.db.profile.file = tg.file
    end
end

local function createText(x, y, lbltext, db, fmt)
    local text = UI.CreateFrame("Frame", "Text", window)
    text:SetPoint("TOPLEFT", window, "TOPLEFT", x + 20, y + 40)
    text:SetWidth(100)
    text:SetHeight(30)
    
    text.label = UI.CreateFrame("Text", "Text label", text)
    text.label:SetText(lbltext)
    text.label:SetPoint("TOPLEFT", text, "TOPLEFT")
    text.label:SetWidth(40)
    
    text.field = UI.CreateFrame("RiftTextfield", "Text field", text)
    text.field:SetPoint("TOPLEFT", text.label, "TOPRIGHT")
    text.field:SetWidth(30)
    text.field:SetHeight(30)
    text.field:SetBackgroundColor(0, 0, 0)
    local lastText
    text.field.Event.TextfieldChange = function()
        local txt = text.field:GetText()
        local match = txt:match("^" .. Cursor.db.profile[db] .. "$")
        if not match then
            text.button:SetEnabled(true)
        else
            text.button:SetEnabled(false)
        end
    end
    
    text.button = UI.CreateFrame("RiftButton", "Text button", text)
    text.button:SetPoint("TOPLEFT", text.field, "TOPRIGHT")
    text.button.Event.LeftPress = function()
        local txt = text.field:GetText()
        local match = txt:match(fmt)
        if match then
            Cursor.db.profile[db] = tonumber(txt)
        else
            text.field:SetText(Cursor.db.profile[db] .. "")
        end
        text.button:SetEnabled(false)
    end
    text.button:SetEnabled(false)
    text.button:SetText("Ok")
    return text
end

local size = 30
local light_blue = createToggle(15, 15, "light_blue.png")
local light_green = createToggle(15+size*1, 15, "light_green.png")
local light_purple = createToggle(15+size*2, 15, "light_purple.png")
local light_red = createToggle(15+size*3, 15, "light_red.png")
local light_white = createToggle(15+size*4, 15, "light_white.png")
local dark_blue = createToggle(15+size*5, 15, "dark_blue.png")
local dark_green = createToggle(15+size*6, 15, "dark_green.png")
local dark_purple = createToggle(15+size*7, 15, "dark_purple.png")
local dark_red = createToggle(15+size*8, 15, "dark_red.png")

local points_text = createText(15, 30*2, "Points", "points", "%d")
local size_text = createText(15, 30*3, "Size", "size", "%d")

local ccf = UI.CreateFrame("Frame", "colorchanger", window)

local handler = function(r, g, b)
    Cursor.db.profile.r = r
    Cursor.db.profile.g = g
    Cursor.db.profile.b = b
end

local colorChooser = Library.ColorChooser.CreateWidget(ccf, handler, 5, false)
colorChooser:SetPoint("TOPLEFT", points_text, "BOTTOMLEFT", 0, 48)

local cb = UI.CreateFrame("RiftCheckbox", "Checkbox", window)
cb:SetPoint("TOPLEFT", colorChooser, "BOTTOMLEFT")
cb.Event.CheckboxChange = function()
    if cb:GetChecked() then
        Cursor.db.profile.applyBackground = true
    else 
        Cursor.db.profile.applyBackground = false
    end
end

local isLoaded
local function playerLoaded(units)
    if isLoaded then return end
    isLoaded = true
    for k, v in pairs(units) do
        if v == "player" then
            Cursor_SavedVariables = Cursor_SavedVariables or {}
            Cursor.db = Cursor:InitializeDB(Cursor_SavedVariables, defaults)

            cb:SetChecked(Cursor.db.profile.applyBackground)

            points_text.field:SetText(Cursor.db.profile.points .. "")
            size_text.field:SetText(Cursor.db.profile.size .. "")
            

            fileMap[Cursor.db.profile.file].checkbox:SetChecked(true)

            
        end
    end
end

table.insert(Event.Unit.Availability.Full, {playerLoaded, "Cursor", "player loaded"})

table.insert(Command.Slash.Register("cursor"), {function (commands) 
    if commands == "config" then
        Cursor:OpenConfig()
    elseif commands == "reset" then
        Cursor.db:ResetDB()
        Cursor.db = Cursor:InitializeDB(Cursor_SavedVariables, defaults)

        fileMap[Cursor.db.profile.file].checkbox:SetChecked(true)
        points_text.field:SetText(Cursor.db.profile.points .. "")
        size_text.field:SetText(Cursor.db.profile.size .. "")       
    elseif commands == "enable" then
        enabled = true
        print("Cursor is now enabled.")
    elseif commands == "disable" then
        enabled = false
        print("Cursor is now disabled.")
    elseif commands == "toggle" then
        enabled = not enabled
        print("Cursor is now " .. (enabled and "enabled." or "disabled."))
    else
        print("Commands are:")
        print("config: Open the configuration dialog.")
        print("reset: Reset the current saved variable profile.")
        print("enable: Enable the cursor trail.")
        print("disable: Disable the cursor trail.")
        print("toggle: Toggle between enabled and disabled.")
    end
end, "Cursor", "Slash command"})
